import json
import pathlib

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from webdriver_manager.chrome import ChromeDriverManager

from .pretty_printer import print_msg
from .wait_and_find import *

timeOutDelay = 30   # increase if you have a slow internet connection

CREDENTIALS_FILE_PATH = pathlib.Path.home() / '.msteams_class_attender'

def login(data, check=False, browser=None):
    if check:
        print_msg("running login test...", "DEBUG")
        opt = Options()
        opt.add_argument("headless")
        browser = webdriver.Chrome(ChromeDriverManager().install(),options=opt)
        browser.get('https://teams.microsoft.com/_#/calendarv2')
    wait_and_find_ele_by_id(browser, 'i0116', timeOutDelay).send_keys(data['username'])      # enter username
    wait_and_find_ele_by_id(browser, 'idSIButton9', timeOutDelay).click()                    # click next
    print_msg("user name entered", "DEBUG")
    wait_and_find_ele_by_id(browser, 'aadTile', timeOutDelay).click()                        # choose organization account
    print_msg("chose school or organization accoutn", "DEBUG")
    wait_and_find_ele_by_id(browser, 'i0118', timeOutDelay).send_keys(data['password'])      # enter password
    wait_and_find_ele_by_id(browser, 'idSIButton9', timeOutDelay).click()                    # click next
    print_msg("password entered", "DEBUG")
    pass_error = wait_and_find_ele_by_id(browser, 'passwordError', timeOutDelay)
    if pass_error is not None:
        raise Exception(pass_error.get_attribute("innerHTML"))
    if not check:
        wait_and_find_ele_by_id(browser, 'idSIButton9', timeOutDelay).click()                    # click yes to stay signed in
    else:
        browser.quit()

def load_credentials(reset=False):
    modified = False
    data = {}
    if not reset:
        try:
            with open(CREDENTIALS_FILE_PATH) as f:
                data = json.load(f)
            print_msg("saved credentials loaded!", "DEBUG")
            if data.get("username") is None:
                data['username'] = input("Please enter your MS Teams username: ")
                modified = True
            if data.get("password") is None:
                data["password"] = input("Please enter your MS Teams password: ")
                modified = True
        except (FileNotFoundError, json.JSONDecodeError):
            print_msg("Failed to load from credentials file!", "DEBUG")
            reset = True
    if (reset):
        login_success = False
        while not login_success:
            data['username'] = input("Please enter your MS Teams username: ")
            data["password"] = input("Please enter your MS Teams password: ")
            try:
                print_msg("verifying username and password, please wait for a few minutes...", "INFO")
                login(data, True)
                login_success = True
                print_msg("authenticated successfully", "INFO")
            except:
                print_msg("authenticated failed", "ERROR")
                continue
            data["minimumParticipants"] = input("Please enter minimum participants to exit the meeting: ")
            response = input("Do you want to interact in the meeting while it is going on? (y/n): ")
            while response.lower() not in ["y", "n"]:
                print_msg("please enter y or n!", "ERROR")
                response = input("Do you want to interact in the meeting while it is going on? (y/n): ")
            if response.lower() == "y":
                data["joinMeetingInBackground"] = False
            else:
                data["joinMeetingInBackground"] = True
        modified = True

    if modified:
        with open(CREDENTIALS_FILE_PATH, "x") as f:
            json.dump(data, f)
        print_msg("Credentials saved to " + str(CREDENTIALS_FILE_PATH), "DEBUG")
    return data



from time import sleep

sleepDelay = 4      # increase if you have a slow internet connection

def wait_and_find_ele_by_id(browser, id, timeout):
    sleep(sleepDelay)
    for i in range(timeout):
        try:
            ele = browser.find_element_by_id(id)
        except:
            sleep(sleepDelay)
        else:
            return ele

def wait_and_find_ele_by_link_text(browser, text, timeout):
    sleep(sleepDelay)
    for i in range(timeout):
        try:
            ele = browser.find_element_by_link_text(text)
        except:
            sleep(sleepDelay)
        else:
            return ele

def wait_and_find_element_by_xpath(browser, xpath, timeout):
    sleep(sleepDelay)
    for i in range(timeout):
        try:
            ele = browser.find_element_by_xpath(xpath)
        except:
            sleep(sleepDelay)
        else:
            return ele

def wait_and_find_elements_by_xpath(browser, xpath, timeout):
    sleep(sleepDelay)
    for i in range(timeout):
        try:
            ele = browser.find_elements_by_xpath(xpath)
        except:
            sleep(sleepDelay)
        else:
            return ele


from datetime import datetime
from os import environ
import pathlib

SUPPORTED_TYPES = ["DEBUG", "INFO", "ERROR", "WARNING"]
PATH_TO_HELP = pathlib.Path(__file__).parents[0] / 'static/help.txt'

def print_msg(msg, type):
    if type not in SUPPORTED_TYPES:
        raise Exception("Unsupported type of message")
    if type == "DEBUG" and not environ.get("DEBUG"):
        return
    print('{date} | {type} : {msg}'.format(
        type=type,
        date=datetime.now(),
        msg=msg
        )
    )

def print_help():
    print("""
mst_autoattend is designed to help you attend classes and sleep well as well!

On the first run the program asks the user for the basic information which is stored in your computer and used in subsequent runs.

mst_autoattend supports the following arguments:

--help - show help
--reset - reset configuration

Please report bugs and request features at https://github.com/thealphadollar/mst_autoattend

""")


import os
import sys
from distutils.core import setup

from setuptools import find_packages, setup

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 6)

def read(file_name):
    """
    Reads data from file for the description parameter of `setup()`.
    Parameters
    ----------
    file_name: str
        path to the README.md file.
    Returns
    -------
    str
        the text contained in the given file.
    """
    with open(file_name, mode='r') as readme:
        return readme.read()


# This check and everything above must remain compatible with Python 2.7.
if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write("""
==========================
Unsupported Python version
==========================
This version requires Python {}.{}, but you're trying to
install it on Python {}.{}.
""".format(*(REQUIRED_PYTHON + CURRENT_PYTHON)))
    sys.exit(1)

setup(name='mst_autoattend',
    version='0.1.2',
    packages=find_packages(),
    description='A tool to attend the MS Team meetings for you!',
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    author='Shivam Kumar Jha',
    author_email='coffee@thealphadollar.me',
    url='https://github.com/thealphadollar/MS-Teams-Class-Attender',
    install_requires=[
    'webdriver_manager==3.2.1',
    'selenium==3.141.0',
    ],
    entry_points={
        'console_scripts': [
            'mst_autoattend=mst_autoattend:main',
        ],
    },
    include_package_data=True
)



import os
import sys
from random import randint
from time import sleep

from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from webdriver_manager.chrome import ChromeDriverManager

from .credentials import load_credentials, login
from .pretty_printer import print_msg, print_help
from .wait_and_find import *

minRandomJoinLeaveWait = 10
maxRandomJoinLeaveWait = 180

timeOutDelay = 30   # increase if you have a slow internet connection
RETRY_WAITOUT = 30
data = None

maxParticipants = curParticipants = 0
minParticipants = 2

opt = Options()
opt.add_argument("--disable-infobars")
opt.add_argument("start-maximized")
opt.add_argument("--disable-extensions")
# Pass the argument 1 to allow and 2 to block
opt.add_experimental_option("prefs", { \
    "profile.default_content_setting_values.media_stream_mic": 1, 
    "profile.default_content_setting_values.media_stream_camera": 1,
    "profile.default_content_setting_values.notifications": 1 
})

browser = None

def checkAndJoinMeeting():
    global maxParticipants, curParticipants
    joins = wait_and_find_elements_by_xpath(browser, '//button[.="Join"]', 3)
    if len(joins) == 0: # no meeting scheduled
        print_msg("no meeting to join", "INFO")
        return
    print_msg("meeting found, joining the latest meeting...", "INFO")
    sleep(randint(minRandomJoinLeaveWait, maxRandomJoinLeaveWait))
    joins[-1].click()   # join the latest meeting scheduled i.e if join buttons for 9 A.M and 10 A.M available, will join 10 A.M
    elem = wait_and_find_element_by_xpath(browser, '//*[@id="page-content-wrapper"]/div[1]/div/calling-pre-join-screen/div/div/div[2]/div[1]/div[2]/div/div/section/div[2]/toggle-button[1]/div/button', timeOutDelay)
    print_msg("turning off camera...", "DEBUG")
    if elem.get_attribute('aria-pressed') == 'true': # turn off camera
        elem.click()
    print_msg("turning off microphone...", "DEBUG")
    elem = wait_and_find_element_by_xpath(browser, '//*[@id="preJoinAudioButton"]/div/button', timeOutDelay)
    if elem.get_attribute('aria-pressed') == 'true': # turn off microphone
        elem.click()
    wait_and_find_element_by_xpath(browser, '//button[.="Join now"]', timeOutDelay).click() # join meeting
    print_msg("meeting joined", "INFO")
    sleep(60*5)
    actions = ActionChains(browser)
    rosterBtn = wait_and_find_element_by_xpath(browser, '//button[@id="roster-button"]', timeOutDelay)
    actions.move_to_element(rosterBtn).click().perform()
    numStr = wait_and_find_elements_by_xpath(browser, '//span[@class="toggle-number"][@ng-if="::ctrl.enableRosterParticipantsLimit"]')
    if len(numStr) >= 2:
        if numStr[1].text[1:-1] != '':
            maxParticipants = curParticipants = int(numStr[1].text[1:-1])

def checkAndEndOrLeaveOrJoinMeeting():
    global maxParticipants, curParticipants
    hangupBtn = wait_and_find_element_by_xpath(browser, '//button[@id="hangup-button"]', 2)
    if hangupBtn != None: # currently in meeting
        print_msg("in a meeting", "DEBUG")
        numStr = wait_and_find_elements_by_xpath(browser, '//span[@class="toggle-number"][@ng-if="::ctrl.enableRosterParticipantsLimit"]')
        if len(numStr) >= 2:
            if numStr[1].text[1:-1] != '':
                curParticipants = int(numStr[1].text[1:-1])
            else :
                actions = ActionChains(browser)
                actions.move_to_element(wait_and_find_element_by_xpath(browser, '//button[@id="roster-button"]', timeOutDelay)).click().perform()
        maxParticipants = max(maxParticipants, curParticipants)
        print_msg("found {} participants in the meeting".format(curParticipants), "DEBUG")
        if curParticipants <= minParticipants and curParticipants != 0:   # leaves the meeting automatically for given condition
            print_msg("leaving meeting as number of participants ({}) is less than minimum set to leave ({})".format(curParticipants, minParticipants), "INFO")
            sleep(randint(minRandomJoinLeaveWait, maxRandomJoinLeaveWait))
            hangupBtn = wait_and_find_element_by_xpath(browser, '//button[@id="hangup-button"]', 3)
            actions = ActionChains(browser)
            actions.move_to_element(hangupBtn).click().perform()
            print_msg("meeting left", "WARNING")
            browser.get('https://teams.microsoft.com/_#/calendarv2')    # open calendar tab
        else :
            return
    else:
        print_msg("not in a meeting, looking for a meeting to join...", "DEBUG")
        maxParticipants = curParticipants = 0
        browser.get('https://teams.microsoft.com/_#/calendarv2')
        checkAndJoinMeeting()

def init(step=0):
    global minParticipants
    print_msg("opening ms teams calendar page...", "DEBUG")
    browser.get('https://teams.microsoft.com/_#/calendarv2')    # open calendar tab in teams
    minParticipants = data.get('minimumParticipants', minParticipants)
    success_step = 0
    # login step
    if step <= 0:
        print_msg("attempting login...", "DEBUG")
        login(data, check=False, browser=browser)
        success_step = 1
    # calendar access step
    if step <= 1:
        print_msg("opening calendar...", "DEBUG")
        wait_and_find_element_by_xpath(browser, '//button[@title="Switch your calendar view"]', timeOutDelay)
        while wait_and_find_element_by_xpath(browser, '//button[@title="Switch your calendar view"]', timeOutDelay).get_attribute('name') != "Week": # change calender work-week view to week view
            wait_and_find_element_by_xpath(browser, '//button[@title="Switch your calendar view"]', timeOutDelay).click()
            wait_and_find_element_by_xpath(browser, '//button[@name="Week"]', timeOutDelay).click()
            print_msg("week view set in calendar", "DEBUG")
        success_step = 2
    return success_step

def main():
    global browser, data
    
    if len(sys.argv) > 1 and sys.argv[1] == "--help":
        print_help()
        return
    
    if len(sys.argv) > 1 and sys.argv[1] == "--reset":
        print_msg("config reset requested by user...", "WARNING")
        data = load_credentials(reset=True)    
    else:
        print_msg("loading credentials from saved file...", "DEBUG")
        data = load_credentials()
    if data.get("joinMeetingInBackground"):
        print_msg("starting in headless mode as per setting...", "INFO")
        opt.add_argument("headless")

    browser = webdriver.Chrome(ChromeDriverManager().install(),options=opt)
 
    step = 0
    while step != 2:
        print_msg("initializing session...", "DEBUG")
        step = init(step)
    else:
        print_msg("initialized successfully!", "INFO")
        while True:
            try:
                print_msg("checking for meeting activity...", "INFO")
                checkAndEndOrLeaveOrJoinMeeting()
            except:
                print_msg("failed to check meeting activity", "WARNING")
                browser.get('https://teams.microsoft.com/_#/calendarv2')    # open calendar tab in teams
            else:
                print_msg("checking teams activity again in {} seconds".format(RETRY_WAITOUT), "INFO")
                sleep(RETRY_WAITOUT)

if __name__ == "__main__":
    main()